#!/bin/bash

if [ -z "$RPM_BUILD_ROOT" -o "$RPM_BUILD_ROOT" = "/" ]; then
	echo "ERROR: no RPM_BUILD_ROOT"
	exit 1
fi
if [ -z "$RPM_TMP_ROOT" -o "$RPM_TMP_ROOT" = "/" ]; then
	echo "ERROR: no RPM_TMP_ROOT"
	exit 1
fi
if [ -z "$TARGET_INSTDIR" -o "$TARGET_INSTDIR" = "/" ]; then
	echo "ERROR: no TARGET_INSTDIR"
	exit 1
fi
if [ -z "$TARGET_ProtoDir" -o "$TARGET_ProtoDir" = "/" ]; then
	echo "ERROR: no TARGET_ProtoDir"
	exit 1
fi
if [ -z "$TARGET_CrossDir" -o "$TARGET_CrossDir" = "/" ]; then
	echo "ERROR: no TARGET_CrossDir"
	exit 1
fi
if [ -z "$TARGET_PRFIX" -o "$TARGET_PRFIX" = "/" ]; then
	echo "ERROR: no TARGET_PRFIX"
	exit 1
fi
#if [ -z "$TARGET_NAME" ]; then
#	echo "ERROR: no TARGET_NAME"
#	exit 1
#fi
if [ -z "$TARGET_LIB" ]; then
	echo "ERROR: no TARGET_LIB"
	exit 1
fi

if [ -z "$MKCROSSDIR" ]; then
  MKCROSSDIR=1
fi

echo "+++ CROSS FILE START +++"
echo ${TARGET_INSTDIR}${TARGET_PRFIX}

#echo $1
#echo $RPM_BUILD_ROOT
#echo $RPM_TMP_ROOT
#echo $TARGET_INSTDIR

# Compress man pages
COMPRESS="gzip -9 -n"
COMPRESS_EXT=.gz

function cross_strip_static() {
  for f in `find "$RPM_BUILD_ROOT/tmp" -type f -a -exec file {} \; | \
	grep 'current ar archive' | \
	sed -n -e 's/^\(.*\):[ 	]*current ar archive/\1/p'`; do
	${TARGET_NAME-}strip -g "$f"
	echo "-g ${f#$RPM_BUILD_ROOT/tmp/*}"
  done
}

function cross_strip_shared() {
    for f in `find "$RPM_BUILD_ROOT/tmp" -type f -a -exec file {} \; | \
	grep ' shared object,' | \
	sed -n -e 's/^\(.*\):[ 	].*ELF.*, not stripped/\1/p'`; do
	${TARGET_NAME-}strip --strip-unneeded "$f" || :
	echo "-unneeded ${f#$RPM_BUILD_ROOT/tmp/*}"
    done
}

function cross_strip_bin() {
    for f in `find "$RPM_BUILD_ROOT/tmp" -type f \( -perm -0100 -or -perm -0010 -or -perm -0001 \) -exec file {} \; | \
	grep -v ' shared object,' | \
	sed -n -e 's/^\(.*\):[ 	].*ELF.*, not stripped/\1/p'`; do
	${TARGET_NAME-}strip -s "$f" || :
	echo "-s ${f#$RPM_BUILD_ROOT/tmp/*}"
    done
}

function cross_strip_comment() {
    for f in `find "$RPM_BUILD_ROOT/tmp" -type f \( -perm -0100 -or -perm -0010 -or -perm -0001 \) -exec file {} \; | \
	sed -n -e 's/^\(.*\):[ 	].*ELF.*, stripped/\1/p'`; do
	note="-R .note"
	if ${TARGET_NAME-}objdump -h $f | grep '^[ 	]*[0-9]*[ 	]*.note[ 	]' -A 1 | \
		grep ALLOC >/dev/null; then
		note=
	fi
	${TARGET_NAME-}strip -R .comment $note "$f" || :
	echo "-R .comment $note ${f#$RPM_BUILD_ROOT/tmp/*}"
    done
}

function man_compress() {

  for d in $RPM_TMP_ROOT$TARGET_PRFIX/man*; do

    [ -d $d ] || continue
    for f in `find $d -type f`
    do
        [ -f "$f" ] || continue
	[ "`basename $f`" = "dir" ] && continue

	case "$f" in
	 *.gz|*.Z)    gunzip  $f; b=`echo $f | sed -e 's/\.\(gz\|Z\)$//'`;;
	 *.bz2)       bunzip2 $f; b=`echo $f | sed -e 's/\.bz2$//'`;;
	 *.xz|*.lzma) unxz    $f; b=`echo $f | sed -e 's/\.\(xz\|lzma\)$//'`;;
	 *) b=$f;;
	esac

#    echo "man_compress $b"

	$COMPRESS $b </dev/null 2>/dev/null || {
	    inode=`ls -i $b | awk '{ print $1 }'`
	    others=`find $d -type f -inum $inode`
	    if [ -n "$others" ]; then
		for afile in $others ; do
		    [ "$afile" != "$b" ] && rm -f $afile
		done
		$COMPRESS -f $b
		for afile in $others ; do
		    [ "$afile" != "$b" ] && ln $b$COMPRESS_EXT $afile$COMPRESS_EXT
		done
	    else
		$COMPRESS -f $b
	    fi
	}
    done

    for f in `find $d -type l`
    do
	l=`ls -l $f | sed -e 's/.* -> //' -e 's/\.\(gz\|Z\|bz2\|xz\|lzma\)$//'`
	rm -f $f
	b=`echo $f | sed -e 's/\.\(gz\|Z\|bz2\|xz\|lzma\)$//'`
	ln -sf $l$COMPRESS_EXT $b$COMPRESS_EXT
    done
  done
}

if test ! -d $RPM_BUILD_ROOT/usr; then
  mkdir -p $RPM_BUILD_ROOT/usr
fi
if test ! -d $TARGET_INSTDIR -a $MKCROSSDIR != "0" ; then
  mkdir -p $TARGET_INSTDIR
fi
#if test ! -d $TARGET_INSTDIR/lib; then
#  mkdir -p $TARGET_INSTDIR/lib
#fi
if test ! -d $TARGET_INSTDIR/$TARGET_LIB/pkgconfig -a $MKCROSSDIR != "0" ; then
  mkdir -p $TARGET_INSTDIR/$TARGET_LIB/pkgconfig
fi

if test ! -d $TARGET_ProtoDir; then
  mkdir -p $TARGET_ProtoDir
fi
#if test ! -d $TARGET_ProtoDir/lib; then
#  mkdir -p $TARGET_ProtoDir/lib
#fi
if test ! -d $TARGET_ProtoDir/$TARGET_LIB/pkgconfig; then
  mkdir -p $TARGET_ProtoDir/$TARGET_LIB/pkgconfig
fi

echo "+++ strip_bin +++"
cross_strip_bin
echo "+++ strip_shared +++"
cross_strip_shared
echo "+++ strip_comment +++"
cross_strip_comment
echo "+++ strip_static +++"
cross_strip_static
echo "+++ man_compress +++"
man_compress

echo "+++ copy data +++"




pushd $RPM_TMP_ROOT$TARGET_PRFIX >/dev/null
for i in *; do
  if test $i = "$TARGET_LIB"; then
    for h in $TARGET_LIB/*; do
      if test x$h = x"$TARGET_LIB/pkgconfig"; then

	pushd $TARGET_LIB/pkgconfig >/dev/null
	  for pc in *; do

	    echo -n >$pc.tmp
	    cat $pc | while read linedat; do
	      case "$linedat" in
	      sdkdir=*)
		echo "sdkdir=$TARGET_PRFIX/include/xorg" >>$pc.tmp
	      ;;
	      *)
		echo $linedat >>$pc.tmp
	      ;;
	      esac
	    done
	    rm $pc
	    mv $pc.tmp $pc

	    if test $MKCROSSDIR != "0" ; then
	      echo -n >$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
	    fi
	    echo -n >$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
	    cat $pc | while read linedat; do

	      case "$linedat" in
	      includedir=*)
		if test $MKCROSSDIR != "0" ; then
		  echo "includedir=$TARGET_CrossDir/include" >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		fi
		echo "includedir=$TARGET_ProtoDir/include" >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
	      ;;
	      libdir=*)
		if test $MKCROSSDIR != "0" ; then
		  echo "libdir=$TARGET_CrossDir/$TARGET_LIB" >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		fi
		echo "libdir=$TARGET_ProtoDir/$TARGET_LIB" >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
	      ;;
	      includex11dir=*)
		echo "--> $linedat"
		if test $MKCROSSDIR != "0" ; then
		  echo "includex11dir=$TARGET_CrossDir/include/X11" >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		fi
		echo "includex11dir=$TARGET_ProtoDir/include/X11" >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
	      ;;
	      pythondir=*)
		if test -z "$TARGET_PYTHONDIR" ; then
		  echo "ERROR: ------ no TARGET_PYTHONDIR !!!! ------ ERROR"
		  if test $MKCROSSDIR != "0" ; then
		    echo "$linedat" >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		  fi
		  echo "$linedat" >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
		else 
		  if test $MKCROSSDIR != "0" ; then
		    echo "pythondir=$TARGET_CrossDir/lib$TARGET_PYTHONDIR" >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		  fi
		  echo "pythondir=$TARGET_ProtoDir/lib$TARGET_PYTHONDIR" >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
		fi
	      ;;
	      xcbincludedir=*)
		if test $MKCROSSDIR != "0" ; then
		  echo "xcbincludedir=$TARGET_CrossDir/$TARGET_LIB/xcb" >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		fi
		echo "xcbincludedir=$TARGET_ProtoDir/$TARGET_LIB/xcb" >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
	      ;;
	      sdkdir=*)
		if test $MKCROSSDIR != "0" ; then
		  echo "sdkdir=$TARGET_CrossDir/include/xorg" >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		fi
		echo "sdkdir=$TARGET_ProtoDir/include/xorg" >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
	      ;;
	      *)
		if test $MKCROSSDIR != "0" ; then
		  echo $linedat >>$TARGET_INSTDIR/$TARGET_LIB/pkgconfig/$pc
		fi
		echo $linedat >>$TARGET_ProtoDir/$TARGET_LIB/pkgconfig/$pc
	      ;;
	      esac

	    done
	  done
	popd >/dev/null

      else
	if test $MKCROSSDIR != "0" ; then
	  cp --parents -ra $h $TARGET_INSTDIR
	fi
	cp --parents -ra $h $TARGET_ProtoDir/
      fi
    done

  else

    if test $MKCROSSDIR != "0" ; then
      cp --parents -ra $i $TARGET_INSTDIR
    fi
    cp --parents -ra $i $TARGET_ProtoDir
  fi
done
popd >/dev/null

cp -ra $RPM_TMP_ROOT/usr $RPM_BUILD_ROOT/
if test -d $RPM_TMP_ROOT/etc ; then
  mv $RPM_TMP_ROOT/etc $RPM_BUILD_ROOT/
fi
if test -d $RPM_TMP_ROOT/var ; then
  mv $RPM_TMP_ROOT/var $RPM_BUILD_ROOT/
fi

rm -r $RPM_TMP_ROOT

if test -d $RPM_BUILD_ROOT$TARGET_PRFIX/$TARGET_LIB/aclocal ;then
  mkdir -p $RPM_BUILD_ROOT/usr/share/aclocal
  cp -a $RPM_BUILD_ROOT$TARGET_PRFIX/$TARGET_LIB/aclocal/* $RPM_BUILD_ROOT/usr/share/aclocal/
  rm -r $RPM_BUILD_ROOT$TARGET_PRFIX/$TARGET_LIB/aclocal
fi


echo "+++ CROSS FILE END +++"
